@@ -149,9 +149,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 
 	private LearningStrategy<D> learner;
 
-	protected final AssertingClauseGenerator analyzer;
-
-	private volatile boolean undertimeout;
+	protected volatile boolean undertimeout;
 
 	private long timeout = Integer.MAX_VALUE;
 
@@ -173,16 +171,21 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 
 	private final IVecInt learnedLiterals = new VecInt();
 
-	private boolean verbose = true;
+	private boolean verbose = false;
 
 	private String prefix = "c ";
 
 	protected IVecInt dimacs2internal(IVecInt in) {
 		__dimacs_out.clear();
 		__dimacs_out.ensure(in.size());
+		int p;
 		for (int i = 0; i < in.size(); i++) {
-			assert (in.get(i) != 0);
-			__dimacs_out.unsafePush(voc.getFromPool(in.get(i)));
+		    p = in.get(i);
+			if (p == 0) {
+				throw new IllegalArgumentException(
+						"0 is not a valid variable identifier");
+			}
+			__dimacs_out.unsafePush(voc.getFromPool(p));
 		}
 		return __dimacs_out;
 	}
@@ -191,19 +194,15 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	 * creates a Solver without LearningListener. A learningListener must be
 	 * added to the solver, else it won't backtrack!!! A data structure factory
 	 * must be provided, else it won't work either.
-	 * 
-	 * @param acg
-	 *            an asserting clause generator
 	 */
 
-	public Solver(AssertingClauseGenerator acg, LearningStrategy<D> learner,
-			D dsf, IOrder order, RestartStrategy restarter) {
-		this(acg, learner, dsf, new SearchParams(), order, restarter);
+	public Solver(LearningStrategy<D> learner, D dsf, IOrder order,
+			RestartStrategy restarter) {
+		this(learner, dsf, new SearchParams(), order, restarter);
 	}
 
-	public Solver(AssertingClauseGenerator acg, LearningStrategy<D> learner,
-			D dsf, SearchParams params, IOrder order, RestartStrategy restarter) {
-		analyzer = acg;
+	public Solver(LearningStrategy<D> learner, D dsf, SearchParams params,
+			IOrder order, RestartStrategy restarter) {
 		this.learner = learner;
 		this.order = order;
 		this.params = params;
@@ -473,7 +472,11 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 
 	private final IVecInt moutLearnt = new VecInt();
 
-	public void analyze(Constr confl, Pair results) {
+	/**
+	 * @throws TimeoutException
+	 *             if the timeout is reached during conflict analysis.
+	 */
+	public void analyze(Constr confl, Pair results) throws TimeoutException {
 		assert confl != null;
 
 		final boolean[] seen = mseen;
@@ -486,7 +489,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 			seen[i] = false;
 		}
 
-		analyzer.initAnalyze();
+		int counter = 0;
 		int p = ILits.UNDEFINED;
 
 		outLearnt.push(ILits.UNDEFINED);
@@ -505,7 +508,8 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 				if (!seen[q >> 1]) {
 					seen[q >> 1] = true;
 					if (voc.getLevel(q) == decisionLevel()) {
-						analyzer.onCurrentDecisionLevelLiteral(q);
+						counter++;
+						order.updateVarAtDecisionLevel(q);
 					} else if (voc.getLevel(q) > 0) {
 						// only literals assigned after decision level 0 part of
 						// the explanation
@@ -523,7 +527,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 			} while (!seen[p >> 1]);
 			// seen[p.var] indique que p se trouve dans outLearnt ou dans
 			// le dernier niveau de d?cision
-		} while (analyzer.clauseNonAssertive(confl));
+		} while (--counter > 0);
 
 		outLearnt.set(0, p ^ 1);
 		simplifier.simplify(outLearnt);
@@ -555,11 +559,20 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		if (assumps.size() == 0) {
 			return null;
 		}
+		if (trailLim.last() == trail.size()) {
+			// conflict detected when assuming a value
+			trailLim.pop();
+		}
 		final boolean[] seen = mseen;
 		final IVecInt outLearnt = moutLearnt;
 		final IVecInt preason = mpreason;
 
 		outLearnt.clear();
+		if (trailLim.size() == 0) {
+			// conflict detected on unit clauses
+			return outLearnt;
+		}
+
 		assert outLearnt.size() == 0;
 		for (int i = 0; i < seen.length; i++) {
 			seen[i] = false;
@@ -570,14 +583,21 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		}
 
 		int p = ILits.UNDEFINED;
-		while (confl == null) {
+		while (confl == null && trail.size() > 0) {
 			p = trail.last();
 			confl = voc.getReason(p);
 			undoOne();
-			if (trail.size() <= trailLim.last()) {
+			if (confl == null && trailLim.size() > 0
+					&& p == (conflictingLiteral ^ 1)) {
+				outLearnt.push(toDimacs(p));
+			}
+			if (trailLim.size() > 0 && trail.size() <= trailLim.last()) {
 				trailLim.pop();
 			}
 		}
+		if (confl == null) {
+			return outLearnt;
+		}
 		do {
 
 			preason.clear();
@@ -660,6 +680,23 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		}
 	};
 
+	public final ISimplifier EXPENSIVE_SIMPLIFICATION_WLONLY = new ISimplifier() {
+
+		/**
+         * 
+         */
+		private static final long serialVersionUID = 1L;
+
+		public void simplify(IVecInt conflictToReduce) {
+			expensiveSimplificationWLOnly(conflictToReduce);
+		}
+
+		@Override
+		public String toString() {
+			return "Expensive reason simplification specific for WL data structure"; //$NON-NLS-1$
+		}
+	};
+
 	private ISimplifier simplifier = NO_SIMPLIFICATION;
 
 	/**
@@ -761,6 +798,59 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	// visiting literals at a level that cannot be removed.
 	//
 	private boolean analyzeRemovable(int p) {
+		assert voc.getReason(p) != null;
+		ILits lvoc = voc;
+		IVecInt lanalyzestack = analyzestack;
+		IVecInt lanalyzetoclear = analyzetoclear;
+		lanalyzestack.clear();
+		lanalyzestack.push(p);
+		final boolean[] seen = mseen;
+		int top = lanalyzetoclear.size();
+		while (lanalyzestack.size() > 0) {
+			int q = lanalyzestack.last();
+			assert lvoc.getReason(q) != null;
+			Constr c = lvoc.getReason(q);
+			lanalyzestack.pop();
+			for (int i = 0; i < c.size(); i++) {
+				int l = c.get(i);
+				if (lvoc.isFalsified(l) && !seen[var(l)]
+						&& lvoc.getLevel(l) != 0) {
+					if (lvoc.getReason(l) == null) {
+						for (int j = top; j < lanalyzetoclear.size(); j++)
+							seen[lanalyzetoclear.get(j) >> 1] = false;
+						lanalyzetoclear.shrink(lanalyzetoclear.size() - top);
+						return false;
+					}
+					seen[l >> 1] = true;
+					lanalyzestack.push(l);
+					lanalyzetoclear.push(l);
+				}
+			}
+		}
+
+		return true;
+	}
+
+	// Taken from MiniSAT 1.14
+	private void expensiveSimplificationWLOnly(IVecInt conflictToReduce) {
+		// Simplify conflict clause (a lot):
+		//
+		int i, j;
+		// (maintain an abstraction of levels involved in conflict)
+		analyzetoclear.clear();
+		conflictToReduce.copyTo(analyzetoclear);
+		for (i = 1, j = 1; i < conflictToReduce.size(); i++)
+			if (voc.getReason(conflictToReduce.get(i)) == null
+					|| !analyzeRemovableWLOnly(conflictToReduce.get(i)))
+				conflictToReduce.moveTo(j++, i);
+		conflictToReduce.shrink(i - j);
+		stats.reducedliterals += (i - j);
+	}
+
+	// Check if 'p' can be removed.' min_level' is used to abort early if
+	// visiting literals at a level that cannot be removed.
+	//
+	private boolean analyzeRemovableWLOnly(int p) {
 		assert voc.getReason(p) != null;
 		analyzestack.clear();
 		analyzestack.push(p);
@@ -771,9 +861,9 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 			assert voc.getReason(q) != null;
 			Constr c = voc.getReason(q);
 			analyzestack.pop();
-			for (int i = 0; i < c.size(); i++) {
+			for (int i = 1; i < c.size(); i++) {
 				int l = c.get(i);
-				if (voc.isFalsified(l) && !seen[var(l)] && voc.getLevel(l) != 0) {
+				if (!seen[var(l)] && voc.getLevel(l) != 0) {
 					if (voc.getReason(l) == null) {
 						for (int j = top; j < analyzetoclear.size(); j++)
 							seen[analyzetoclear.get(j) >> 1] = false;
@@ -796,25 +886,24 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
      * 
      */
 	protected void undoOne() {
-		// recupere le dernier litteral affecte
+		// gather last assigned literal
 		int p = trail.last();
 		assert p > 1;
 		assert voc.getLevel(p) >= 0;
 		int x = p >> 1;
-		// desaffecte la variable
+		// unassign variable
 		voc.unassign(p);
 		voc.setReason(p, null);
 		voc.setLevel(p, -1);
-		// met a jour l'heuristique
+		// update heuristics value
 		order.undo(x);
-		// depile le litteral des affectations
+		// remove literal from the trail
 		trail.pop();
-		// met a jour les contraintes apres desaffectation du litteral :
-		// normalement, il n'y a rien a faire ici pour les prouveurs de type
-		// Chaff??
+		// update constraints on backtrack.
+		// not used if the solver uses watched literals.
 		IVec<Undoable> undos = voc.undos(p);
 		assert undos != null;
-		while (undos.size() > 0) {
+		for (int size = undos.size(); size > 0; size--) {
 			undos.last().undo(p);
 			undos.pop();
 		}
@@ -846,32 +935,53 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		claInc *= CLAUSE_RESCALE_FACTOR;
 	}
 
+	private final IVec<Propagatable> watched = new Vec<Propagatable>();
+
 	/**
 	 * @return null if not conflict is found, else a conflicting constraint.
 	 */
 	public Constr propagate() {
-		while (qhead < trail.size()) {
-			stats.propagations++;
-			int p = trail.get(qhead++);
-			slistener.propagating(toDimacs(p), null);
-			order.assignLiteral(p);
+		IVec<Propagatable> lwatched = watched;
+		IVecInt ltrail = trail;
+		ILits lvoc = voc;
+		SolverStats lstats = stats;
+		IOrder lorder = order;
+		SearchListener lslistener = slistener;
+		// ltrail.size() changes due to propagation
+		// cannot cache that value.
+		while (qhead < ltrail.size()) {
+			lstats.propagations++;
+			int p = ltrail.get(qhead++);
+			lslistener.propagating(toDimacs(p), null);
+			lorder.assignLiteral(p);
 			// p is the literal to propagate
 			// Moved original MiniSAT code to dsfactory to avoid
 			// watches manipulation in counter Based clauses for instance.
 			assert p > 1;
-			IVec<Propagatable> watched = dsfactory.getWatchesFor(p);
-
-			final int size = watched.size();
+			lwatched.clear();
+			lvoc.watches(p).moveTo(lwatched);
+			final int size = lwatched.size();
 			for (int i = 0; i < size; i++) {
-				stats.inspects++;
-				if (!watched.get(i).propagate(this, p)) {
+				lstats.inspects++;
+				// try shortcut
+				// shortcut = shortcuts.get(i);
+				// if (shortcut != ILits.UNDEFINED && voc.isSatisfied(shortcut))
+				// {
+				// voc.watch(p, watched.get(i), shortcut);
+				// stats.shortcuts++;
+				// continue;
+				// }
+				if (!lwatched.get(i).propagate(this, p)) {
 					// Constraint is conflicting: copy remaining watches to
 					// watches[p]
 					// and return constraint
-					dsfactory.conflictDetectedInWatchesFor(p, i);
-					qhead = trail.size(); // propQ.clear();
+					final int sizew = lwatched.size();
+					for (int j = i + 1; j < sizew; j++) {
+						lvoc.watch(p, lwatched.get(j));
+					}
+					qhead = ltrail.size(); // propQ.clear();
 					// FIXME enlever le transtypage
-					return (Constr) watched.get(i);
+					return (Constr) lwatched.get(i);
 				}
 			}
 		}
@@ -1010,7 +1120,11 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 					return Lbool.FALSE;
 				}
 				// analyze conflict
-				analyze(confl, analysisResult);
+				try {
+					analyze(confl, analysisResult);
+				} catch (TimeoutException e) {
+					return Lbool.UNDEFINED;
+				}
 				assert analysisResult.backtrackLevel < decisionLevel();
 				backjumpLevel = Math.max(analysisResult.backtrackLevel,
 						rootLevel);
@@ -1035,6 +1149,9 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	protected void analyzeAtRootLevel(Constr conflict) {
 	}
 
+	private final IVecInt implied = new VecInt();
+	private final IVecInt decisions = new VecInt();
+
 	/**
      * 
      */
@@ -1048,6 +1165,11 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 				if (!voc.isUnassigned(p)) {
 					model[index++] = voc.isSatisfied(p) ? i : -i;
 					fullmodel[i - 1] = voc.isSatisfied(p);
+					if (voc.getReason(p) == null) {
+						decisions.push(model[index - 1]);
+					} else {
+						implied.push(model[index - 1]);
+					}
 				}
 			}
 		}
@@ -1055,6 +1177,36 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		cancelUntil(rootLevel);
 	}
 
+	public int[] primeImplicant() {
+		IVecInt currentD = new VecInt(decisions.size());
+		decisions.copyTo(currentD);
+		IVecInt assumptions = new VecInt(implied.size() + decisions.size());
+		implied.copyTo(assumptions);
+		decisions.copyTo(assumptions);
+		IVecInt prime = new VecInt(assumptions.size());
+		implied.copyTo(prime);
+		for (int i = 0; i < currentD.size(); i++) {
+			int p = currentD.get(i);
+			assumptions.remove(p);
+			assumptions.push(-p);
+			try {
+				if (isSatisfiable(assumptions)) {
+					assumptions.pop();
+					assumptions.push(-p);
+				} else {
+					prime.push(p);
+					assumptions.pop();
+					assumptions.push(p);
+				}
+			} catch (TimeoutException e) {
+				throw new IllegalStateException("Should not timeout here", e);
+			}
+		}
+		int[] implicant = new int[prime.size()];
+		prime.copyTo(implicant);
+		return implicant;
+	}
+
 	public boolean model(int var) {
 		if (var <= 0 || var > nVars()) {
 			throw new IllegalArgumentException(
@@ -1075,7 +1227,6 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	}
 
 	protected void reduceDB() {
-		sortOnActivity();
 		stats.reduceddb++;
 		learnedConstraintsDeletionStrategy.reduce(learnts);
 		System.gc();
@@ -1142,14 +1293,75 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		void onConflictAnalysis(Constr reason);
 	}
 
+	public final LearnedConstraintsDeletionStrategy fixedSize(final int maxsize) {
+		return new LearnedConstraintsDeletionStrategy() {
+
+			private static final long serialVersionUID = 1L;
+			private final ConflictTimer aTimer = new ConflictTimerAdapter(
+					maxsize) {
+
+				private static final long serialVersionUID = 1L;
+
+				@Override
+				void run() {
+					needToReduceDB = true;
+				}
+			};
+
+			public void reduce(IVec<Constr> learnedConstrs) {
+				int i, j, k;
+				for (i = j = k = 0; i < learnts.size()
+						&& learnts.size() - k > maxsize; i++) {
+					Constr c = learnts.get(i);
+					if (c.locked() || c.size() == 2) {
+						learnts.set(j++, learnts.get(i));
+					} else {
+						c.remove(Solver.this);
+						k++;
+					}
+				}
+				for (; i < learnts.size(); i++) {
+					learnts.set(j++, learnts.get(i));
+				}
+				if (verbose) {
+					System.out.println(getLogPrefix()
+							+ "cleaning " + (learnts.size() - j) //$NON-NLS-1$
+							+ " clauses out of " + learnts.size()); //$NON-NLS-1$ //$NON-NLS-2$
+					System.out.flush();
+				}
+				learnts.shrinkTo(j);
+			}
+
+			public void onConflictAnalysis(Constr reason) {
+				// TODO Auto-generated method stub
+
+			}
+
+			public void onConflict(Constr outLearnt) {
+				// TODO Auto-generated method stub
+
+			}
+
+			@Override
+			public String toString() {
+				return "Fixed size (" + maxsize
+						+ ") learned constraints deletion strategy";
+			}
+
+			public void init() {
+			}
+
+			public ConflictTimer getTimer() {
+				return aTimer;
+			}
+		};
+	}
+
 	/**
 	 * @since 2.1
 	 */
 	public final LearnedConstraintsDeletionStrategy memory_based = new LearnedConstraintsDeletionStrategy() {
 
-		/**
-		 * 
-		 */
 		private static final long serialVersionUID = 1L;
 
 		final long memorybound = Runtime.getRuntime().freeMemory() / 10;
@@ -1169,6 +1381,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		};
 
 		public void reduce(IVec<Constr> learnedConstrs) {
+			sortOnActivity();
 			int i, j;
 			for (i = j = 0; i < learnts.size() / 2; i++) {
 				Constr c = learnts.get(i);
@@ -1185,6 +1398,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 				System.out.println(getLogPrefix()
 						+ "cleaning " + (learnts.size() - j) //$NON-NLS-1$
 						+ " clauses out of " + learnts.size()); //$NON-NLS-1$ //$NON-NLS-2$
+				System.out.flush();
 			}
 			learnts.shrinkTo(j);
 		}
@@ -1218,9 +1432,6 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	 */
 	public final LearnedConstraintsDeletionStrategy glucose = new LearnedConstraintsDeletionStrategy() {
 
-		/**
-		 * 
-		 */
 		private static final long serialVersionUID = 1L;
 		private int[] flags = new int[0];
 		private int flag = 0;
@@ -1246,9 +1457,20 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 					needToReduceDB = true;
 				}
 			}
+
+			@Override
+			public void reset() {
+				super.reset();
+				nextbound = MAX_CLAUSE;
+				if (nbconflict >= nextbound) {
+					nbconflict = 0;
+					needToReduceDB = true;
+				}
+			}
 		};
 
 		public void reduce(IVec<Constr> learnedConstrs) {
+			sortOnActivity();
 			int i, j;
 			for (i = j = learnedConstrs.size() / 2; i < learnedConstrs.size(); i++) {
 				Constr c = learnedConstrs.get(i);
@@ -1263,6 +1485,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 						.println(getLogPrefix()
 								+ "cleaning " + (learnedConstrs.size() - j) //$NON-NLS-1$
 								+ " clauses out of " + learnedConstrs.size() + " with flag " + flag + "/" + stats.conflicts); //$NON-NLS-1$ //$NON-NLS-2$
+				System.out.flush();
 			}
 			learnts.shrinkTo(j);
 
@@ -1283,7 +1506,8 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 			if (flags.length <= howmany) {
 				flags = new int[howmany + 1];
 			}
-
+			flag = 0;
+			clauseManagement.reset();
 		}
 
 		public void onConflict(Constr constr) {
@@ -1319,7 +1543,6 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	public boolean isSatisfiable(IVecInt assumps, boolean global)
 			throws TimeoutException {
 		Lbool status = Lbool.UNDEFINED;
-		undertimeout = true;
 		final int howmany = voc.nVars();
 		if (mseen.length <= howmany) {
 			mseen = new boolean[howmany + 1];
@@ -1327,6 +1550,8 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		trail.ensure(howmany);
 		trailLim.ensure(howmany);
 		learnedLiterals.ensure(howmany);
+		decisions.clear();
+		implied.clear();
 		timebegin = System.currentTimeMillis();
 		slistener.start();
 		model = null; // forget about previous model
@@ -1387,6 +1612,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		if (timeBasedTimeout) {
 			if (!global || timer == null) {
 				firstTimeGlobal = true;
+				undertimeout = true;
 				TimerTask stopMe = new TimerTask() {
 					@Override
 					public void run() {
@@ -1400,6 +1626,7 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 		} else {
 			if (!global || conflictCount == null) {
 				firstTimeGlobal = true;
+				undertimeout = true;
 				ConflictTimer conflictTimeout = new ConflictTimerAdapter(
 						(int) timeout) {
 					private static final long serialVersionUID = 1L;
@@ -1579,8 +1806,8 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	 */
 	public String toString(String prefix) {
 		StringBuffer stb = new StringBuffer();
-		Object[] objs = { analyzer, dsfactory, learner, params, order,
-				simplifier, restarter, learnedConstraintsDeletionStrategy };
+		Object[] objs = { dsfactory, learner, params, order, simplifier,
+				restarter, learnedConstraintsDeletionStrategy };
 		stb.append(prefix);
 		stb.append("--- Begin Solver configuration ---"); //$NON-NLS-1$
 		stb.append("\n"); //$NON-NLS-1$
@@ -1685,12 +1912,21 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	 * @since 2.1
 	 */
 	public void unset(int p) {
+		// the literal might already been
+		// removed from the trail.
+		if (trail.isEmpty()) {
+			return;
+		}
 		int current = trail.last();
 		while (current != p) {
 			undoOne();
+			if (trail.isEmpty()) {
+				return;
+			}
 			current = trail.last();
 		}
 		undoOne();
+		qhead = trail.size();
 	}
 
 	/**
@@ -1711,7 +1947,9 @@ public class Solver<D extends DataStructureFactory> implements ISolver,
 	 * @since 2.2
 	 */
 	public IVecInt unsatExplanation() {
-		return unsatExplanationInTermsOfAssumptions;
+		IVecInt copy = new VecInt(unsatExplanationInTermsOfAssumptions.size());
+		unsatExplanationInTermsOfAssumptions.copyTo(copy);
+		return copy;
 	}
 
 }
