@@ -69,6 +69,8 @@ public abstract class AbstractSelectorVariablesDecorator extends
 	 */
 	protected boolean[] prevboolmodel;
 
+	protected boolean isSolutionOptimal;
+
 	public AbstractSelectorVariablesDecorator(ISolver solver) {
 		super(solver);
 	}
@@ -105,6 +107,7 @@ public abstract class AbstractSelectorVariablesDecorator extends
 	 */
 	public boolean admitABetterSolution(IVecInt assumps)
 			throws TimeoutException {
+		isSolutionOptimal = false;
 		boolean result = super.isSatisfiable(assumps, true);
 		if (result) {
 			prevboolmodel = new boolean[nVars()];
@@ -112,7 +115,7 @@ public abstract class AbstractSelectorVariablesDecorator extends
 				prevboolmodel[i] = decorated().model(i + 1);
 			}
 			prevfullmodel = super.model();
-			int end = nborigvars - 1;
+			int end = Math.min(prevfullmodel.length, nborigvars) - 1;
 			while (Math.abs(prevfullmodel[end]) > nborigvars)
 				end--;
 			prevmodel = new int[end + 1];
@@ -120,6 +123,8 @@ public abstract class AbstractSelectorVariablesDecorator extends
 				prevmodel[i] = prevfullmodel[i];
 			}
 			calculateObjectiveValue();
+		} else {
+			isSolutionOptimal = true;
 		}
 		return result;
 	}
@@ -135,4 +140,8 @@ public abstract class AbstractSelectorVariablesDecorator extends
 	public boolean model(int var) {
 		return prevboolmodel[var - 1];
 	}
+
+	public boolean isOptimal() {
+		return isSolutionOptimal;
+	}
 }
