@@ -29,13 +29,14 @@ package org.sat4j.tools.xplain;
 
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 import org.sat4j.core.VecInt;
 import org.sat4j.specs.ContradictionException;
 import org.sat4j.specs.IConstr;
-import org.sat4j.specs.IOptimizationProblem;
 import org.sat4j.specs.ISolver;
 import org.sat4j.specs.IVecInt;
 import org.sat4j.specs.IteratorInt;
@@ -43,23 +44,12 @@ import org.sat4j.specs.TimeoutException;
 import org.sat4j.tools.SolverDecorator;
 
 /**
- * An implementation of the QuickXplain algorithm as explained by Ulrich Junker
- * in the following paper:
+ * Explanation framework for SAT4J.
  * 
- * @inproceedings{ junker01:quickxplain:inp, author={Ulrich Junker},
- *                 title={QUICKXPLAIN: Conflict Detection for Arbitrary
- *                 Constraint Propagation Algorithms}, booktitle={IJCAI'01
- *                 Workshop on Modelling and Solving problems with constraints
- *                 (CONS-1)}, year={2001}, month={August}, address={Seattle, WA,
- *                 USA}, url={citeseer.ist.psu.edu/junker01quickxplain.html},
- *                 url={http://www.lirmm.fr/~bessiere/ws_ijcai01/junker.ps.gz} }
+ * The explanation uses selector variables and assumptions.
  * 
- *                 The algorithm has been adapted to work properly in a context
- *                 where we can afford to add a selector variable to each clause
- *                 to enable or disable each constraint.
- * 
- *                 Note that for the moment, QuickXplain does not work properly
- *                 in an optimization setting.
+ * It is based on a two steps method: 1) extraction of a set of assumptions
+ * implying the inconsistency 2) minimization of that set.
  * 
  * @author daniel
  * 
@@ -67,7 +57,8 @@ import org.sat4j.tools.SolverDecorator;
  *            a subinterface to ISolver.
  * @since 2.1
  */
-public class Xplain<T extends ISolver> extends SolverDecorator<T> {
+public class Xplain<T extends ISolver> extends SolverDecorator<T> implements
+		Explainer {
 
 	protected Map<Integer, IConstr> constrs = new HashMap<Integer, IConstr>();
 
@@ -77,7 +68,7 @@ public class Xplain<T extends ISolver> extends SolverDecorator<T> {
 	private boolean pooledVarId = false;
 	private final IVecInt lastClause = new VecInt();
 
-	private static final XplainStrategy XPLAIN_STRATEGY = new QuickXplainStrategy();
+	private MinimizationStrategy xplainStrategy = new QuickXplainStrategy();
 
 	public Xplain(T solver) {
 		super(solver);
@@ -138,18 +129,40 @@ public class Xplain<T extends ISolver> extends SolverDecorator<T> {
 	 */
 	private static final long serialVersionUID = 1L;
 
+	/**
+	 * @since 2.2.4
+	 * @return
+	 * @throws TimeoutException
+	 */
+	private IVecInt explanationKeys() throws TimeoutException {
+		assert !isSatisfiable(assump);
+		ISolver solver = decorated();
+		if (solver instanceof SolverDecorator<?>) {
+			solver = ((SolverDecorator<? extends ISolver>) solver).decorated();
+		}
+		return xplainStrategy.explain(solver, constrs, assump);
+	}
+
+	public int[] minimalExplanation() throws TimeoutException {
+		IVecInt keys = explanationKeys();
+		keys.sort();
+		List<Integer> allKeys = new ArrayList<Integer>(constrs.keySet());
+		Collections.sort(allKeys);
+		int[] model = new int[keys.size()];
+		int i = 0;
+		for (IteratorInt it = keys.iterator(); it.hasNext();) {
+			model[i++] = allKeys.indexOf(it.next()) + 1;
+		}
+		return model;
+	}
+
 	/**
 	 * @since 2.1
 	 * @return
 	 * @throws TimeoutException
 	 */
 	public Collection<IConstr> explain() throws TimeoutException {
-		assert !isSatisfiable(assump);
-		ISolver solver = decorated();
-		if (solver instanceof IOptimizationProblem) {
-			solver = ((SolverDecorator<? extends ISolver>) solver).decorated();
-		}
-		IVecInt keys = XPLAIN_STRATEGY.explain(solver, constrs, assump);
+		IVecInt keys = explanationKeys();
 		Collection<IConstr> explanation = new ArrayList<IConstr>(keys.size());
 		for (IteratorInt it = keys.iterator(); it.hasNext();) {
 			explanation.add(constrs.get(it.next()));
@@ -161,7 +174,7 @@ public class Xplain<T extends ISolver> extends SolverDecorator<T> {
 	 * @since 2.1
 	 */
 	public void cancelExplanation() {
-		XPLAIN_STRATEGY.cancelExplanationComputation();
+		xplainStrategy.cancelExplanationComputation();
 	}
 
 	/**
@@ -252,4 +265,15 @@ public class Xplain<T extends ISolver> extends SolverDecorator<T> {
 		return model;
 	}
 
+	@Override
+	public String toString(String prefix) {
+		System.out.println(prefix + "Explanation (MUS) enabled solver");
+		System.out.println(prefix + xplainStrategy);
+		return super.toString(prefix);
+	}
+
+	public void setMinimizationStrategy(MinimizationStrategy strategy) {
+		xplainStrategy = strategy;
+	}
+
 }
