@@ -684,8 +684,8 @@ public class OPBReader2005 extends Reader implements Serializable {
 		try {
 			parse();
 			return solver;
-                } catch (ContradictionException ce) {
-                        throw ce;
+		} catch (ContradictionException ce) {
+			throw ce;
 		} catch (Exception e) {
 			String message;
 
@@ -695,8 +695,8 @@ public class OPBReader2005 extends Reader implements Serializable {
 			} else {
 				message = e.getMessage();
 			}
-			throw new ParseFormatException(" line " + input.getLineNumber()
-					+ ", " + message);
+			throw new ParseFormatException(" line "
+					+ (input.getLineNumber() + 1) + ", " + message);
 
 		}
 	}
